(function() {
  var assert, connectionListener, eslClient, eslParser, eslRequest, eslResponse, eslServer, net, parse_header_text, querystring, util;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor; child.__super__ = parent.prototype; return child; };

  net = require('net');

  querystring = require('querystring');

  util = require('util');

  assert = require('assert');

  exports.debug = false;

  parse_header_text = function(header_text) {
    var header_lines, headers, line, name, _fn, _i, _len, _ref;
    if (exports.debug) util.log("parse_header_text(" + header_text + ")");
    header_lines = header_text.split("\n");
    headers = {};
    _fn = function(line) {
      var name, value, _ref;
      _ref = line.split(/: /, 2), name = _ref[0], value = _ref[1];
      return headers[name] = value;
    };
    for (_i = 0, _len = header_lines.length; _i < _len; _i++) {
      line = header_lines[_i];
      _fn(line);
    }
    if (((_ref = headers['Reply-Text']) != null ? _ref[0] : void 0) === '%') {
      for (name in headers) {
        headers[name] = querystring.unescape(headers[name]);
      }
    }
    return headers;
  };

  eslParser = (function() {

    function eslParser(socket) {
      this.socket = socket;
      this.body_length = 0;
      this.buffer = "";
    }

    eslParser.prototype.capture_body = function(data) {
      var body;
      this.buffer += data;
      if (this.buffer.length < this.body_length) return;
      body = this.buffer.substring(0, this.body_length);
      this.buffer = this.buffer.substring(this.body_length);
      this.body_length = 0;
      this.process(this.headers, body);
      this.headers = {};
      return this.capture_headers('');
    };

    eslParser.prototype.capture_headers = function(data) {
      var header_end, header_text;
      this.buffer += data;
      header_end = this.buffer.indexOf("\n\n");
      if (header_end < 0) return;
      header_text = this.buffer.substring(0, header_end);
      this.buffer = this.buffer.substring(header_end + 2);
      this.headers = parse_header_text(header_text);
      if (this.headers["Content-Length"]) {
        this.body_length = this.headers["Content-Length"];
        return this.capture_body('');
      } else {
        this.process(this.headers);
        this.headers = {};
        return this.capture_headers('');
      }
    };

    eslParser.prototype.on_data = function(data) {
      if (exports.debug) util.log("on_data(" + data + ")");
      if (this.body_length > 0) {
        return this.capture_body(data);
      } else {
        return this.capture_headers(data);
      }
    };

    eslParser.prototype.on_end = function() {
      if (exports.debug) {
        util.log("Parser: end of stream");
        if (this.buffer.length > 0) {
          return util.log("Buffer is not empty, left over: " + this.buffer);
        }
      }
    };

    return eslParser;

  })();

  eslRequest = (function() {

    function eslRequest(headers, body) {
      this.headers = headers;
      this.body = body;
    }

    return eslRequest;

  })();

  eslResponse = (function() {

    function eslResponse(socket) {
      this.socket = socket;
    }

    eslResponse.prototype.send = function(command, args, cb) {
      var key, value;
      if (exports.debug) {
        util.log(util.inspect({
          command: command,
          args: args
        }));
      }
      this.socket.removeAllListeners('esl_command_reply');
      this.socket.removeAllListeners('esl_api_response');
      if (cb != null) {
        this.socket.on('esl_command_reply', cb);
        this.socket.on('esl_api_response', cb);
      }
      this.socket.write("" + command + "\n");
      if (args != null) {
        for (key in args) {
          value = args[key];
          this.socket.write("" + key + ": " + value + "\n");
        }
      }
      return this.socket.write("\n");
    };

    eslResponse.prototype.on = function(event, listener) {
      return this.socket.on(event, listener);
    };

    eslResponse.prototype.end = function() {
      return this.socket.end();
    };

    eslResponse.prototype.api = function(command, cb) {
      return this.send("api " + command, null, cb);
    };

    eslResponse.prototype.bgapi = function(command, cb) {
      return this.send("bgapi " + command, null, function(req, res) {
        var r, _ref;
        if (cb != null) {
          r = (_ref = res.header['Reply-Text']) != null ? _ref.match(/\+OK Job-UUID: (.+)$/) : void 0;
          return cb(r[1]);
        }
      });
    };

    eslResponse.prototype.event_json = function() {
      var cb, events, _i;
      events = 2 <= arguments.length ? __slice.call(arguments, 0, _i = arguments.length - 1) : (_i = 0, []), cb = arguments[_i++];
      return this.send("event json " + (events.join(' ')), null, cb);
    };

    eslResponse.prototype.nixevent = function() {
      var cb, events, _i;
      events = 2 <= arguments.length ? __slice.call(arguments, 0, _i = arguments.length - 1) : (_i = 0, []), cb = arguments[_i++];
      return this.send("nixevent " + (events.join(' ')), null, cb);
    };

    eslResponse.prototype.noevents = function(cb) {
      return this.send("noevents", null, cb);
    };

    eslResponse.prototype.filter = function(header, value, cb) {
      return this.send("filter " + header + " " + value, null, cb);
    };

    eslResponse.prototype.filter_delete = function(header, value, cb) {
      if (value != null) {
        return this.send("filter " + header + " " + value, null, cb);
      } else {
        return this.send("filter " + header, null, cb);
      }
    };

    eslResponse.prototype.sendevent = function(event_name, args, cb) {
      return this.send("sendevent " + event_name, args, cb);
    };

    eslResponse.prototype.auth = function(password, cb) {
      return this.send("auth " + password, null, cb);
    };

    eslResponse.prototype.connect = function(cb) {
      return this.send("connect", null, cb);
    };

    eslResponse.prototype.linger = function(cb) {
      return this.send("linger", null, cb);
    };

    eslResponse.prototype.exit = function(cb) {
      return this.send("exit", null, cb);
    };

    eslResponse.prototype.log = function(level, cb) {
      var _ref;
      if (typeof level === 'function') {
        _ref = [null, level], level = _ref[0], cb = _ref[1];
      }
      if (level != null) {
        return this.send("log " + level, null, cb);
      } else {
        return this.send("log", null, cb);
      }
    };

    eslResponse.prototype.nolog = function(cb) {
      return this.send("nolog", null, cb);
    };

    eslResponse.prototype.sendmsg_uuid = function(uuid, command, args, cb) {
      var execute_text, options;
      options = args != null ? args : {};
      options['call-command'] = command;
      execute_text = uuid != null ? "sendmsg " + uuid : 'sendmsg';
      return this.send(execute_text, options, cb);
    };

    eslResponse.prototype.sendmsg = function(command, args, cb) {
      return this.sendmsg_uuid(null, command, args, cb);
    };

    eslResponse.prototype.execute_uuid = function(uuid, app_name, app_arg, cb) {
      var options;
      options = {
        'execute-app-name': app_name,
        'execute-app-arg': app_arg
      };
      return this.sendmsg_uuid(uuid, 'execute', options, cb);
    };

    eslResponse.prototype.hangup_uuid = function(uuid, hangup_cause, cb) {
      var options;
      if (hangup_cause == null) hangup_cause = 'NORMAL_UNSPECIFIED';
      options = {
        'hangup-cause': hangup_cause
      };
      return this.sendmsg_uuid(uuid, 'hangup', options, cb);
    };

    eslResponse.prototype.unicast_uuid = function(uuid, args, cb) {
      return this.sendmsg_uuid(uuid, 'unicast', args, cb);
    };

    eslResponse.prototype.execute = function(app_name, app_arg, cb) {
      return this.execute_uuid(null, app_name, app_arg, cb);
    };

    eslResponse.prototype.hangup = function(hangup_cause, cb) {
      return this.hangup_uuid(null, hangup_cause, cb);
    };

    eslResponse.prototype.unicast = function(args, cb) {
      return this.unicast_uuid(null, args, cb);
    };

    return eslResponse;

  })();

  connectionListener = function(socket) {
    var parser;
    socket.setEncoding('ascii');
    parser = new eslParser(socket);
    socket.on('data', function(data) {
      return parser.on_data(data);
    });
    socket.on('end', function() {
      return parser.on_end();
    });
    parser.process = function(headers, body) {
      var event, n, req, res, _i, _len, _ref;
      if (exports.debug) {
        util.log(util.inspect({
          headers: headers,
          body: body
        }));
      }
      switch (headers['Content-Type']) {
        case 'auth/request':
          event = 'esl_auth_request';
          break;
        case 'command/reply':
          event = 'esl_command_reply';
          if (headers['Event-Name'] === 'CHANNEL_DATA') {
            body = headers;
            headers = {};
            _ref = ['Content-Type', 'Reply-Text', 'Socket-Mode', 'Control'];
            for (_i = 0, _len = _ref.length; _i < _len; _i++) {
              n = _ref[_i];
              headers[n] = body[n];
              delete body[n];
            }
          }
          break;
        case 'text/event-json':
          try {
            body = JSON.parse(body);
          } catch (error) {
            util.log("JSON " + error + " in " + body);
            return;
          }
          event = 'esl_event';
          break;
        case 'text/event-plain':
          body = parse_header_text(body);
          event = 'esl_event';
          break;
        case 'log/data':
          event = 'esl_log_data';
          break;
        case 'text/disconnect-notice':
          event = 'esl_disconnect_notice';
          break;
        case 'api/response':
          event = 'esl_api_response';
          break;
        default:
          event = headers['Content-Type'];
      }
      req = new eslRequest(headers, body);
      res = new eslResponse(socket);
      if (exports.debug) {
        util.log(util.inspect({
          event: event,
          req: req,
          res: res
        }));
      }
      return socket.emit(event, req, res);
    };
    return socket.emit('esl_connect', new eslResponse(socket));
  };

  eslServer = (function() {

    __extends(eslServer, net.Server);

    function eslServer(requestListener) {
      this.on('connection', function(socket) {
        socket.on('esl_connect', requestListener);
        return connectionListener(socket);
      });
      eslServer.__super__.constructor.call(this);
    }

    return eslServer;

  })();

  exports.createServer = function(requestListener) {
    return new eslServer(requestListener);
  };

  eslClient = (function() {

    __extends(eslClient, net.Socket);

    function eslClient() {
      this.on('connect', function() {
        return connectionListener(this);
      });
      eslClient.__super__.constructor.call(this);
    }

    return eslClient;

  })();

  exports.createClient = function() {
    return new eslClient();
  };

  exports.createCallServer = function() {
    var Unique_ID, server;
    Unique_ID = 'Unique-ID';
    return server = new eslServer(function(res) {
      var _this = this;
      if (exports.debug) {
        util.log("Incoming connection");
        util.log(util.inspect(res));
      }
      return res.connect(function(req, res) {
        var channel_data, unique_id;
        channel_data = req.body;
        unique_id = channel_data[Unique_ID];
        if (exports.debug) util.log("Incoming call UUID = " + unique_id);
        res.on('esl_disconnect_notice', function(req, res) {
          if (exports.debug) util.log("Received ESL disconnection notice");
          switch (req.headers['Content-Disposition']) {
            case 'linger':
              return res.exit();
            case 'disconnect':
              return res.end();
          }
        });
        res.on('force_disconnect', function() {
          util.log('Hangup call');
          return this.bgapi("uuid_kill " + unique_id);
        });
        res.on('esl_event', function(req, res) {
          req.channel_data = channel_data;
          req.unique_id = unique_id;
          return server.emit(req.body['Event-Name'], req, res);
        });
        return res.linger(function(req, res) {
          return res.filter(Unique_ID, unique_id, function(req, res) {
            return res.event_json('ALL', function(req, res) {
              req.channel_data = channel_data;
              req.unique_id = unique_id;
              return server.emit('CONNECT', req, res);
            });
          });
        });
      });
    });
  };

}).call(this);
